# coding: utf-8
import os
import datetime
import pathlib
from jinjaql.factory import JinJAQL, SnaqlException
from jinjaql.convertors import SnaqlGuardException
try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestGuards(unittest.TestCase):

    def setUp(self):
        self.sql_root = os.path.abspath(os.path.dirname(__file__))
        self.snaql = JinJAQL(pathlib.Path(self.sql_root, 'queries'))

    def test_guards_normal(self):
        news_queries = self.snaql.load_queries('news.sql')
        date = datetime.datetime.now()
        self.assertEqual(
            news_queries.select_by_id(
                news_id=10,
                date_from=date,
                rating='5.6',
            ), (
                "SELECT *\n FROM news\n "
                "WHERE id = 10\n "
                "AND creation_date >= "
                "'{0.year:04}-{0.month:02}-{0.day:02} "
                "{0.hour:02}:{0.minute:02}:{0.second:02}.{0.microsecond:06}'\n "
                "AND rating >= 5.6"
            ).format(date)
        )

    def test_guard_integer_exc(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertRaises(
            SnaqlGuardException,
            news_queries.select_by_id,
            news_id='mouse'
        )

    def test_guard_datetime_exc(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertRaises(
            SnaqlGuardException,
            news_queries.select_by_id,
            date_from='tomorrow'
        )

    def test_guard_case_exc(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertRaises(
            SnaqlGuardException,
            news_queries.get_news,
            sort_order='REV'
        )

    def test_guard_case(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertEqual(
            news_queries.get_news(sort_order='ASC'),
            'SELECT *\n FROM news\n  ORDER BY creation_date ASC'
        )

    def test_guard_regexp(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertEqual(
            news_queries.select_by_slug(slug='latest_news'),
            "SELECT *\n FROM news\n WHERE slug = 'latest_news'"
        )

    def test_guard_exc(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertRaises(
            SnaqlGuardException,
            news_queries.select_by_slug,
            slug='latest news',
        )

    def test_conditions_sugar_without_context(self):
        news_queries = self.snaql.load_queries('news.sql')
        response = news_queries.get_news(conditions=[
            news_queries.cond_ids_in_news,
            news_queries.cond_date_from_news,
            news_queries.cond_date_to_news,
        ])
        self.assertEqual(response, 'SELECT *\n FROM news')

    def test_conditions_sugar_with_context(self):
        news_queries = self.snaql.load_queries('news.sql')
        today = datetime.date.today()
        context = {
            'ids': (1, 2, 3),
            'date_from': today,
            'sort_order': 'ASC',
        }
        response = news_queries.get_news(conditions=[
            news_queries.cond_ids_in_news,
            news_queries.cond_date_from_news,
            news_queries.cond_date_to_news,
        ], **context)
        self.assertEqual(
            response, (
                "SELECT *\n FROM news\n WHERE id IN (1, 2, 3) "
                "AND creation_date >= "
                "'{0.year:04}-{0.month:02}-{0.day:02}' "
                "ORDER BY creation_date ASC"
            ).format(today)
        )

    def test_conditions_sugar_wrong_context(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertRaises(
            SnaqlException,
            news_queries.get_news,
            conditions=[news_queries.cond_date_to_another_news]
        )

    def test_conditions_sugar_outside_context(self):
        news_queries = self.snaql.load_queries('news.sql')
        self.assertRaises(
            SnaqlException,
            news_queries.cond_date_to_another_news,
        )



# coding: utf-8
import os
import pathlib
from jinjaql.factory import JinJAQL, SnaqlException
try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestMigrations(unittest.TestCase):

    def setUp(self):
        self.sql_root = os.path.abspath(os.path.dirname(__file__))
        self.snaql = JinJAQL(pathlib.Path(self.sql_root, 'queries'))

    def test_guard_integer_exc(self):
        migrate_queries = self.snaql.load_queries('migrations.sql')
        call_order = [fn.func_name for fn in migrate_queries.ordered_blocks]
        # All of these solutions are valid
        possible_solutions = (
            [
                'create_templates',
                'create_clusters',
                'create_dummy',
                'create_flavors',
                'create_nodes',
            ],
            [
                'create_templates',
                'create_dummy',
                'create_flavors',
                'create_nodes',
                'create_clusters',
            ],
            [
                'create_templates',
                'create_dummy',
                'create_clusters',
                'create_nodes',
                'create_clusters',
            ],
            [
                'create_templates',
                'create_dummy',
                'create_clusters',
                'create_flavors',
                'create_nodes',
            ],
            [
                'create_templates',
                'create_dummy',
                'create_flavors',
                'create_clusters',
                'create_nodes',
            ],
        )
        self.assertTrue(call_order in possible_solutions)



# coding: utf-8
import os
import datetime
import pathlib
from jinjaql.factory import JinJAQL
try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestUseCases(unittest.TestCase):

    def setUp(self):
        self.sql_root = os.path.abspath(os.path.dirname(__file__))
        self.snaql = JinJAQL(pathlib.Path(self.sql_root, 'queries'))

    def test_usual_case(self):
        users_queries = self.snaql.load_queries('users.sql')
        self.assertEqual(
            users_queries.users_by_country(), (
                "SELECT count(*) AS count\n "
                "FROM user\n "
                "WHERE country_code = 'UA'"
            )
        )
        self.assertEqual(
            users_queries.select_all(), (
                "SELECT * "
                "FROM user"
            )
        )

    def test_subrendering(self):
        users_queries = self.snaql.load_queries('users.sql')

        self.assertEqual(
            users_queries.users_count_cond(), (
                "SELECT count(*) AS count\n "
                "FROM user\n "
            )
        )
        context = {'by_country': True, 'country_code': 42}
        self.assertEqual(
            users_queries.users_count_cond(**context), (
                "SELECT count(*) AS count\n "
                "FROM user\n "
                "WHERE country_code = 42"
            )
        )

    def test_complex_subrendering(self):
        users_queries = self.snaql.load_queries('users.sql')

        context = {'users_ids': [1, 2, 3]}
        self.assertEqual(
            users_queries.users_select_cond(**context), (
                "SELECT *\n "
                "FROM user\n "
                "WHERE user_id IN (1, 2, 3)"
            )
        )

    def test_escaping(self):
        users_queries = self.snaql.load_queries('users.sql')

        context = {'user_name': "semirook"}
        self.assertEqual(
            users_queries.users_escaping(**context), (
                "SELECT *\n FROM user\n "
                "WHERE user_name = 'semirook'"
            )
        )

    def test_clean_env(self):
        users_queries = self.snaql.load_queries('users.sql')
        self.assertEqual(
            users_queries.select_all(), (
                "SELECT * "
                "FROM user"
            )
        )
        self.assertFalse(self.snaql.jinja_env.sql_params)

    def test_multiple_ns(self):
        users_queries = self.snaql.load_queries('users.sql')
        news_queries = self.snaql.load_queries('news.sql')
        self.assertEqual(
            users_queries.select_all(), (
                "SELECT * "
                "FROM user"
            )
        )
        self.assertEqual(
            news_queries.select_all(), (
                "SELECT *\n "
                "FROM news"
            )
        )

    def test_guards(self):
        news_queries = self.snaql.load_queries('news.sql')
        news_queries.select_by_id(
            news_id=10,
            date_from=datetime.datetime.now(),
            rating='5.6',
        )

    def test_simple_tmpl(self):
        queries = self.snaql.load_queries('integration.sql')
        result_list = queries.simple_tmpl(var=[1, 2, 3])
        self.assertEqual(result_list, '[1, 2, 3]')

        result_dict = queries.simple_tmpl(var={'a': 1, 'b': 2})
        self.assertTrue(
            result_dict in ("{'a': 1, 'b': 2}", "{'b': 2, 'a': 1}")
        )

    def test_tmpl_with_comment(self):
        users_queries = self.snaql.load_queries('users.sql')
        abc = users_queries.select_all_with_comment()



# coding: utf-8
import os
import datetime
import pathlib
from schema import Schema, And, Use, SchemaError
from jinjaql.factory import JinJAQL
from jinjaql.convertors import guard_date, escape_string
try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestSchemaCases(unittest.TestCase):

    def setUp(self):
        self.sql_root = os.path.abspath(os.path.dirname(__file__))
        self.snaql = JinJAQL(pathlib.Path(self.sql_root, 'queries'))

    def test_usual_case(self):
        news_queries = self.snaql.load_queries('unsafe_news.sql')
        now = datetime.datetime.utcnow()
        schema = Schema({
            'news_id': And(Use(int), lambda i: i > 0),
            'rating': And(Use(float), lambda r: r > 0),
            'date_from': And(Use(guard_date)),
        })
        context = {
            'news_id': '123',
            'date_from': now,
            'rating': 4.22,
        }
        news_queries.select_by_id(schema=schema, **context)

    def test_usual_string_case(self):
        news_queries = self.snaql.load_queries('unsafe_news.sql')
        schema = Schema({
            'slug': And(Use(escape_string), Use(lambda s: "`%s_slug`" % s)),
        })
        context = {
            'slug': 'cool',
        }
        result = news_queries.select_by_slug(schema=schema, **context)
        self.assertEqual(
            result, 'SELECT *\n FROM news\n WHERE slug = `cool_slug`'
        )

    def test_wrong_case(self):
        news_queries = self.snaql.load_queries('unsafe_news.sql')
        now = datetime.datetime.utcnow()
        schema = Schema({
            'news_id': And(Use(int), lambda i: i > 0),
            'rating': And(Use(float), lambda r: r > 0),
            'date_from': And(Use(guard_date)),
        })
        context = {
            'news_id': '0',
            'date_from': now,
            'rating': 4.22,
        }
        with self.assertRaises(SchemaError):
            news_queries.select_by_id(schema=schema, **context)



def default(query_string: str, connection_string: str):
    """
    :param query_string: This is the raw query string that the SQL engine would use to return the data
    :param connection: This is the connection string that comes from the SQL template that can be used for the SQL query
    :return: The default will return the string w/o changes to keep the default SNAQL behavior, but any additional
    would return data (ie DataFrame)
    """
    return query_string




# coding: utf-8
import os
import copy
import collections
import types
import sys
import functools
from collections import namedtuple
import pathlib

from jinja2 import nodes
from jinja2 import Environment, TemplateNotFound, FileSystemLoader
from jinja2.ext import Extension
from jinja2.loaders import split_template_path
from jinja2.utils import open_if_exists
from schema import Schema

from jinjaql.convertors import (
    guard_bool,
    guard_case,
    guard_date,
    guard_datetime,
    guard_float,
    guard_integer,
    guard_regexp,
    guard_string,
    guard_time,
    guard_timedelta,
)

import jinjaql.engine as engine


PY = sys.version_info
PY3K = PY >= (3, 0, 0)


class RawFileSystemLoader(FileSystemLoader):

    def get_source(self, environment, template):
        pieces = split_template_path(template)
        for searchpath in self.searchpath:
            filename = os.path.join(searchpath, *pieces)
            f = open_if_exists(filename)
            if f is None:
                continue
            try:
                contents = f.read().decode(self.encoding)
            finally:
                f.close()

            mtime = os.path.getmtime(filename)
            # Need to save original raw template before compilation
            environment.sql_params.setdefault('raws', {}).update({
                template: [c.strip() for c in contents.splitlines()]
            })

            def uptodate():
                try:
                    return os.path.getmtime(filename) == mtime
                except OSError:
                    return False
            return contents, filename, uptodate

        raise TemplateNotFound(template)


class JinjaSQLExtension(Extension):
    tags = set(['sql', 'query'])

    def parse(self, parser):
        lineno = next(parser.stream).lineno
        expr = parser.parse_expression()
        args = [expr]
        kwargs = [nodes.Keyword('func', expr)]
        if parser.stream.skip_if('comma'):
            # Optional 'note' for function docstring
            if (
                parser.stream.current.type == 'name' and
                parser.stream.current.value in (
                    'note',
                    'cond_for',
                    'depends_on',
                    'connection_string', #TODO
                )
            ):
                stream_type = parser.stream.current.value
                next(parser.stream)
                parser.stream.expect('assign')
                # Depends meta is always a list
                if stream_type == 'depends_on':
                    c_expr = parser.parse_list()
                else:
                    c_expr = parser.parse_expression()
                args.append(c_expr)
                kwargs.append(nodes.Keyword(stream_type, c_expr))

        body = parser.parse_statements(
            ['name:endsql', 'name:endquery'], drop_needle=True
        )
        raw_template = self.environment.sql_params['raws'][parser.name]

        # Lines range of original raw template
        raw_lines = slice(lineno, parser.stream.current.lineno-1)
        self.environment.sql_params.setdefault('funcs', {}).update({
            expr.value: {'raw_sql': '\n '.join(raw_template[raw_lines])}
        })
        call_node = nodes.Call(
            self.attr('_sql_process', lineno=lineno),
            args, kwargs, None, None
        )
        return nodes.CallBlock(call_node, [], [], body)

    def _sql_process(self, *args, **kwargs):
        caller = kwargs['caller']
        raw_sql = '\n '.join(x.strip() for x in caller().split('\n') if x)
        if 'cond_for' in kwargs:
            origin = (
                self.environment.sql_params['funcs'].get(kwargs['cond_for'])
            )
            if origin:
                origin.setdefault('conds', []).append(kwargs['cond_for'])

        origin = self.environment.sql_params['funcs'].get(kwargs['func'])
        origin.update({
            'sql': raw_sql,
            'note': kwargs.get('note'),
            'is_cond': 'cond_for' in kwargs,
            'depends_on': kwargs.get('depends_on', []),
            'connection_string': kwargs.get('connection_string'),#TODO
            'node': None,
        })
        if origin['is_cond']:
            origin['cond_for'] = kwargs['cond_for']

        return raw_sql


class SnaqlDepNode(object):

    def __init__(self, name):
        self.name = name
        self.edges = []

    def add_edge(self, node):
        self.edges.append(node)

    def __str__(self):
        return '<SnaqlDepNode %s>' % self.name

    def __repr__(self):
        return '<SnaqlDepNode %s>' % self.name


class SnaqlException(Exception):
    pass


class JinJAQL(object):

    def __init__(
            self,
            folder_path: pathlib.Path,
            engine=engine.default,
            cache=False,
    ):
        folder_path = pathlib.Path(folder_path)
        self.jinja_env = Environment(
            trim_blocks=True,
            extensions=[JinjaSQLExtension],
            loader=RawFileSystemLoader(folder_path)
        )
        self.jinja_env.filters.update({
            'guards.string': guard_string,
            'guards.integer': guard_integer,
            'guards.datetime': guard_datetime,
            'guards.date': guard_date,
            'guards.float': guard_float,
            'guards.timedelta': guard_timedelta,
            'guards.time': guard_time,
            'guards.case': guard_case,
            'guards.regexp': guard_regexp,
            'guards.bool': guard_bool,
        })
        self.jinja_env.extend(sql_params={})
        self._engine = engine
        self._cache = cache

    def gen_func(self, name, meta_struct, env):

        def subrender_cond(owner_name, cond_func, context):
            if (
                isinstance(cond_func, collections.abc.Callable) and
                cond_func.is_cond
            ):
                cond_struct = meta_struct['funcs'][cond_func.func_name]
                if cond_struct['cond_for'] != owner_name:
                    raise SnaqlException(
                        '"%s" is not proper condition for "%s"' % (
                            cond_func.func_name,
                            owner_name
                        )
                    )
                cond_tmpl = env.from_string(
                    meta_struct['funcs'][cond_func.func_name]['raw_sql']
                )
                return cond_tmpl.render(**context).strip()
            return cond_func

        def fn(**kwargs):
            if meta_struct['funcs'][name]['is_cond']:
                raise SnaqlException((
                    '"%s" is condition for "%s" and can not '
                    'be rendered outside of it\'s scope'
                ) % (name, meta_struct['funcs'][name]['cond_for']))

            if kwargs:
                for point, val in kwargs.items():
                    maybe_cond_sql = subrender_cond(name, val, kwargs)
                    if maybe_cond_sql:
                        kwargs[point] = maybe_cond_sql
                    if (
                        isinstance(val, collections.abc.Iterable) and
                        not isinstance(
                            val, (str if PY3K else types.StringTypes, dict)
                        )
                    ):
                        val = [subrender_cond(name, v, kwargs) for v in val]
                        kwargs[point] = [v for v in val if v]

                if 'schema' in kwargs and isinstance(kwargs['schema'], Schema):
                    validation_schema = kwargs.pop('schema')
                    kwargs = validation_schema.validate(kwargs)

                sql_tmpl = (
                    env.from_string(meta_struct['funcs'][name]['raw_sql'])
                )
                return self._engine(
                    query_string=sql_tmpl.render(**kwargs).strip(),
                    connection_string=meta_struct['funcs'][name]['connection_string'],
                ) #TODO

            return meta_struct['funcs'][name]['sql']

        fn.__doc__ = meta_struct['funcs'][name]['note']
        fn.is_cond = meta_struct['funcs'][name]['is_cond']
        fn.func_name = str(name)
        if self._cache:
            cache_fn = functools.lru_cache()(fn)
            return cache_fn
        else:
            return fn

    def gen_dep_graph(self, node, accum):
        for edge in node.edges:
            if edge not in accum:
                self.gen_dep_graph(edge, accum)

        accum.append(node)

        return accum

    def load_queries(self, sql_path):
        template = self.jinja_env.get_template(sql_path)
        template.render()

        factory_methods = {}
        meta_struct = copy.deepcopy(self.jinja_env.sql_params)
        blocks = set(meta_struct['funcs'])

        node = SnaqlDepNode('root')

        for name, block in meta_struct['funcs'].items():
            # Dependency graph building
            block['node'] = block['node'] or SnaqlDepNode(name)
            for dep in block['depends_on']:
                if dep not in blocks:
                    raise SnaqlException(
                        '"%s" block not found in "%s"' % (dep, sql_path)
                    )
                if meta_struct['funcs'][dep]['node'] is None:
                    meta_struct['funcs'][dep]['node'] = SnaqlDepNode(dep)

                block['node'].add_edge(meta_struct['funcs'][dep]['node'])

            node.add_edge(block['node'])

            fn = self.gen_func(name, meta_struct, self.jinja_env)
            factory_methods[name] = fn

        edges_accum = []
        graph = self.gen_dep_graph(node, edges_accum)
        graph.pop()  # root node

        factory_methods['ordered_blocks'] = [
            factory_methods[n.name]
            for n in graph
        ]

        factory = namedtuple('SQLFactory', factory_methods.keys())
        struct = factory(*factory_methods.values())
        self.jinja_env.sql_params.clear()

        return struct



# coding: utf-8
import re
import collections


ESCAPE_REGEX = re.compile(r"[\0\n\r\032\'\"\\]")
ESCAPE_MAP = {
    '\0': '\\0', '\n': '\\n', '\r': '\\r', '\032': '\\Z',
    '\'': '\\\'', '"': '\\"', '\\': '\\\\'
}


class SnaqlGuardException(Exception):
    pass


def escape_string(value):
    return ESCAPE_REGEX.sub(lambda match: ESCAPE_MAP.get(match.group(0)), value)


def guard_string(value):
    if not value:
        return "''"

    return "'%s'" % escape_string(value)


def guard_bool(value):
    return 1 if value else 0


def guard_integer(value):
    if not value:
        return value
    try:
        return int(value)
    except (TypeError, ValueError) as e:
        raise SnaqlGuardException(e.args[0])


def guard_datetime(obj):
    if not obj:
        return obj

    try:
        if obj.microsecond:
            fmt = (
                "'{0.year:04}-{0.month:02}-{0.day:02} "
                "{0.hour:02}:{0.minute:02}:{0.second:02}.{0.microsecond:06}'"
            )
        else:
            fmt = (
                "'{0.year:04}-{0.month:02}-{0.day:02} "
                "{0.hour:02}:{0.minute:02}:{0.second:02}'"
            )
        return fmt.format(obj)
    except (AttributeError, ValueError) as e:
        raise SnaqlGuardException(e.args[0])


def guard_date(obj):
    if not obj:
        return obj

    fmt = "'{0.year:04}-{0.month:02}-{0.day:02}'"

    try:
        return fmt.format(obj)
    except (AttributeError, ValueError) as e:
        raise SnaqlGuardException(e.args[0])


def guard_float(value):
    if not value:
        return value

    try:
        value = float(value)
        return "%.15g" % value
    except (TypeError, ValueError) as e:
        raise SnaqlGuardException(e.args[0])


def guard_timedelta(obj):
    if not obj:
        return obj

    try:
        seconds = int(obj.seconds) % 60
        minutes = int(obj.seconds // 60) % 60
        hours = int(obj.seconds // 3600) % 24 + int(obj.days) * 24
        if obj.microseconds:
            fmt = "'{0:02d}:{1:02d}:{2:02d}.{3:06d}'"
        else:
            fmt = "'{0:02d}:{1:02d}:{2:02d}'"

        return fmt.format(hours, minutes, seconds, obj.microseconds)
    except (AttributeError, ValueError) as e:
        raise SnaqlGuardException(e.args[0])


def guard_time(obj):
    if not obj:
        return obj

    try:
        if obj.microsecond:
            fmt = "'{0.hour:02}:{0.minute:02}:{0.second:02}.{0.microsecond:06}'"
        else:
            fmt = "'{0.hour:02}:{0.minute:02}:{0.second:02}'"

        return fmt.format(obj)
    except (AttributeError, ValueError) as e:
        raise SnaqlGuardException(e.args[0])


def guard_case(value, items=None):
    if not value:
        return value

    items = items or set()
    if not isinstance(items, collections.abc.Iterable):
        raise SnaqlGuardException('Guard items are not iterable')

    items = set(items)
    if value not in items:
        raise SnaqlGuardException(
            '%s expected, "%s" is bad value' % (', '.join(items), value)
        )
    return value


def guard_regexp(value, regexp):
    if not value:
        return value

    if re.match(regexp, value):
        return value
    else:
        raise SnaqlGuardException(
            '"%s" does not match "%s" expression' % (value, regexp)
        )



# coding: utf-8
import os
import sys
from shutil import rmtree
from setuptools import setup, find_packages, Command


DESCRIPTION = 'SNAQL (Templated SQL) with the ability to pull data.'
HERE_PATH = os.path.dirname(os.path.abspath(__file__))
VERSION = '0.1.1'


try:
    with open(os.path.join(HERE_PATH, 'README.md')) as f:
        long_description = '\n' + f.read()
except IOError:
    long_description = DESCRIPTION


# class UploadCommand(Command):
#     """Support setup.py upload."""
#
#     description = 'Build and publish the package.'
#     user_options = []
#
#     @staticmethod
#     def status(s):
#         """Prints things in bold."""
#         print('\033[1m{0}\033[0m'.format(s))
#
#     def initialize_options(self):
#         pass
#
#     def finalize_options(self):
#         pass
#
#     def run(self):
#         try:
#             self.status('Removing previous builds…')
#             rmtree(os.path.join(HERE_PATH, 'dist'), ignore_errors=True)
#         except OSError:
#             pass
#
#         self.status('Building Source and Wheel (universal) distribution…')
#         os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))
#
#         self.status('Uploading the package to PyPI via Twine…')
#         os.system('twine upload dist/*')
#
#         self.status('Pushing git tags…')
#         os.system('git tag v{0}'.format(VERSION))
#         os.system('git push --tags')
#
#         sys.exit()


setup(
    name='jinjaql',
    version=VERSION,
    author='Richard Foley and David Smit (based on the work of Roman Zaiev)',
    author_email='david.d.smit@gmail.com',
    packages=find_packages(),
    license='MIT',
    url='https://github.com/RichFoley/jinjaql',
    description='Transparant *QL usage without ORM',
    long_description=DESCRIPTION,
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=[
        'Jinja2>=2.9.5',
        'schema>=0.6.5',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    # cmdclass={
    #     'upload': UploadCommand,
    # },
    exclude_package_data={"": ["*GKN*"]}
)



# coding: utf-8
import os
import shutil
import sqlite3
import pathlib
from jinjaql.factory import JinJAQL

try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestIntegration(unittest.TestCase):

    def setUp(self):
        self.sql_root = os.path.abspath(os.path.dirname(__file__))
        self.snaql = JinJAQL(pathlib.Path(self.sql_root, 'queries'))
        self.sqlite_folder = os.path.join(self.sql_root, 'data')
        self.sqlite_path = os.path.join(self.sqlite_folder, 'test_db.sqlite')
        if not os.path.exists(self.sqlite_folder):
            os.makedirs(self.sqlite_folder)
        if not os.path.exists(self.sqlite_path):
            open(self.sqlite_path, 'a').close()

    def tearDown(self):
        shutil.rmtree(self.sqlite_folder)

    def test_req(self):
        queries = self.snaql.load_queries('integration.sql')
        conn = sqlite3.connect(self.sqlite_path)
        conn.execute(queries.create_artists())
        query = queries.insert_artist(**{
            'id': 1,
            'name': 'Lana Del Rey',
            'age': 30,
            'instrument': 'voice',
            'creation_date': '2015-10-13',
        })
        conn.execute(query)
        conn.commit()
        conn.close()

        conn = sqlite3.connect(self.sqlite_path)
        query = queries.get_artists(id=1)
        response = conn.execute(query)

        self.assertEqual(response.fetchone(), ('Lana Del Rey', 30, 'voice'))
        conn.close()